#!/usr/bin/env python
"""
_Mask_

Basic Mask object that can be used to restrict the run/lumi/event range in a
job in two ways:

- Inclusive: The stuff within the mask is processed
- Exclusive: The stuff outside of the mask is processed

"""

from builtins import range
from WMCore.DataStructs.Run import Run


class Mask(dict):
    """
    _Mask_
    """

    def __init__(self, **kwargs):
        dict.__init__(self, **kwargs)
        self.inclusive = True
        self.setdefault("inclusivemask", True)
        self.setdefault("FirstEvent", None)
        self.setdefault("LastEvent", None)
        self.setdefault("FirstLumi", None)
        self.setdefault("LastLumi", None)
        self.setdefault("FirstRun", None)
        self.setdefault("LastRun", None)
        self.setdefault("runAndLumis", {})

    def setMaxAndSkipEvents(self, maxEvents, skipEvents):
        """
        _setMaxAndSkipEvents_

        Set FirstEvent & LastEvent fields as max & skip events

        """

        self['FirstEvent'] = skipEvents
        if maxEvents is not None:
            self['LastEvent'] = skipEvents + maxEvents

        return

    def setMaxAndSkipLumis(self, maxLumis, skipLumi):
        """
        _setMaxAndSkipLumis

        Set the Maximum number of lumi sections and the starting point

        """

        self['FirstLumi'] = skipLumi
        self['LastLumi'] = skipLumi + maxLumis

        return

    def setMaxAndSkipRuns(self, maxRuns, skipRun):
        """
        _setMaxAndSkipRuns

        Set the Maximum number of runss and the starting point

        """

        self['FirstRun'] = skipRun
        self['LastRun'] = skipRun + maxRuns

        return

    def getMaxEvents(self):
        """
        _getMaxEvents_

        return maxevents setting

        """
        if self['LastEvent'] is None or self['FirstEvent'] is None:
            return None
        return self['LastEvent'] - self['FirstEvent'] + 1

    def getMax(self, keyType=None):
        """
        _getMax_

        returns the maximum number of runs/events/etc of the type of the type string
        """
        if 'First%s' % (keyType) not in self:
            return None
        if self['First%s' % (keyType)] is None or self['Last%s' % (keyType)] is None:
            return None
        return self['Last%s' % (keyType)] - self['First%s' % (keyType)] + 1

    def addRun(self, run):
        """
        _addRun_

        Add a run object
        """
        run.lumis.sort()
        firstLumi = run.lumis[0]
        lastLumi = run.lumis[0]
        for lumi in run.lumis:
            if lumi <= lastLumi + 1:
                lastLumi = lumi
            else:
                self.addRunAndLumis(run.run, lumis=[firstLumi, lastLumi])
                firstLumi = lumi
                lastLumi = lumi
        self.addRunAndLumis(run.run, lumis=[firstLumi, lastLumi])
        return

    def addRunWithLumiRanges(self, run, lumiList):
        """
        _addRunWithLumiRanges_

        Add to runAndLumis with call signature
        addRunWithLumiRanges(run=run, lumiList = [[start1,end1], [start2, end2], ...]
        """
        self['runAndLumis'][run] = lumiList
        return

    def addRunAndLumis(self, run, lumis=None):
        """
        _addRunAndLumis_

        Add runs and lumis directly
        TODO: The name of this function is a little misleading. If you pass a list of lumis
              it ignores the content of the list and adds a range based on the max/min in
              the list. Missing lumis in the list are ignored.

        NOTE: If the new run/lumi range overlaps with the pre-existing lumi ranges in the
              mask, no attempt is made to merge these together.  This can result in a mask
              with duplicate lumis.
        """
        lumis = lumis or []
        if not isinstance(lumis, list):
            lumis = list(lumis)

        if run not in self['runAndLumis']:
            self['runAndLumis'][run] = []

        self['runAndLumis'][run].append([min(lumis), max(lumis)])

        return

    def getRunAndLumis(self):
        """
        _getRunAndLumis_

        Return list of active runs and lumis
        """

        return self['runAndLumis']

    def runLumiInMask(self, run, lumi):
        """
        _runLumiInMask_

        See if a particular runLumi is in the mask
        """

        if self['runAndLumis'] == {}:
            # Empty dictionary
            # ALWAYS TRUE
            return True

        if run not in self['runAndLumis']:
            return False

        for pair in self['runAndLumis'][run]:
            # Go through each max and min pair
            if pair[0] <= lumi and pair[1] >= lumi:
                # Then the lumi is bracketed
                return True

        return False

    def filterRunLumisByMask(self, runs):
        """
        _filterRunLumisByMask_

        Pass a Mask a list of run objects, get back a list of
        run objects that correspond to the actual mask allowed values
        """
        if self['runAndLumis'] == {}:
            # Empty dictionary
            # ALWAYS TRUE
            return runs

        runDict = {}
        for r in runs:
            if r.run in runDict:
                runDict[r.run].extendLumis(r.lumis)
            else:
                runDict[r.run] = r

        maskRuns = set(self["runAndLumis"].keys())
        passedRuns = set([r.run for r in runs])
        filteredRuns = maskRuns.intersection(passedRuns)

        newRuns = set()
        for runNumber in filteredRuns:
            maskLumis = set()
            for pair in self["runAndLumis"][runNumber]:
                if pair[0] == pair[1]:
                    maskLumis.add(pair[0])
                else:
                    maskLumis = maskLumis.union(list(range(pair[0], pair[1] + 1)))

            filteredLumis = set(runDict[runNumber].lumis).intersection(maskLumis)
            if len(filteredLumis) > 0:
                filteredLumiEvents = [(lumi, runDict[runNumber].getEventsByLumi(lumi)) for lumi in filteredLumis]
                newRuns.add(Run(runNumber, *filteredLumiEvents))

        return newRuns



"""
Patterns module provides set of CS patterns
"""
import re

class Singleton(type):
    """Implementation of Singleton class"""
    _instances = {}
    def __call__(cls, *args, **kwargs):
        if cls not in cls._instances:
            cls._instances[cls] = \
                    super(Singleton, cls).__call__(*args, **kwargs)
        return cls._instances[cls]


def getDomainName(urlStr):
    """
    Given a URL string, return the domain name.
    :param urlStr: URL string
    :return: a string with the domain name (e.g. "cmsweb-prod")
    """
    domainPattern = re.compile(r'https?://([^/]+)\.cern\.ch')
    match = domainPattern.search(urlStr)
    return match.group(1) if match else ""



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Workflow object representing how DQMHarvest workflows have to be dealt
with in the input data placement (MSTransferor)
"""

from WMCore.MicroService.MSTransferor.DataStructs.Workflow import Workflow
from WMCore.Services.Rucio.RucioUtils import GROUPING_ALL


class DQMHarvestWorkflow(Workflow):
    """
    Class to represent a DQMHarvest workflow in the context
    of input data placement in MSTransferor
    """

    def getInputData(self):
        """
        Returns all the primary data that has to be locked and
        transferred with Rucio.

        :return: a tuple with a list of block names and an integer
                with their total size in bytes
        """
        inputContainer = [self.getInputDataset()]
        totalBlockSize = sum([blockInfo['blockSize'] for blockInfo in self.getPrimaryBlocks().values()])
        # the whole container must be locked
        return inputContainer, totalBlockSize

    def getRucioGrouping(self):
        """
        Returns the rucio rule grouping for DQMHarvest workflows,
        which must hold all the input data under the same RSE in
        order to harvest full stats.

        :return: a string with the required DID grouping
        """
        return GROUPING_ALL



#!/usr/bin/env python
"""
_WMBSBase_

Generic methods used by all of the WMBS classes.
"""




from builtins import object
import threading
import copy

from WMCore.Database.Transaction import Transaction
from WMCore.DAOFactory import DAOFactory

try:
    from contextlib import contextmanager
except (ImportError, NameError):
    pass

class WMConnectionBase(object):
    """
    Generic db connection and transaction methods used by all of the WMCore classes.
    """
    def __init__(self, daoPackage, logger = None, dbi = None):
        """
        ___init___

        Initialize all the database connection attributes and the logging
        attritbutes.  Create a DAO factory for given daoPackage as well. Finally,
        check to see if a transaction object has been created.  If none exists,
        create one but leave the transaction closed.
        """
        myThread = threading.currentThread()
        if logger:
            self.logger = logger
        else:
            self.logger = myThread.logger
        if dbi:
            self.dbi = dbi
        else:
            self.dbi = myThread.dbi

        self.daofactory = DAOFactory(package = daoPackage,
                                     logger = self.logger,
                                     dbinterface = self.dbi)

        if "transaction" not in dir(myThread):
            myThread.transaction = Transaction(self.dbi)

        return

    def getDBConn(self):
        """
        _getDBConn_

        Retrieve the database connection that is associated with the current
        dataabase transaction.
        It transaction exists, it will return connection
        which that transaction belong to.
        This won't create the transaction if it doesn't exist, it will just return
        None.
        """
        myThread = threading.currentThread()

        if "transaction" not in dir(myThread):
            return None

        return myThread.transaction.conn

    def beginTransaction(self):
        """
        _beginTransaction_

        Begin a database transaction if one does not already exist.
        """
        myThread = threading.currentThread()

        if "transaction" not in dir(myThread):
            myThread.transaction = Transaction(self.dbi)
            return False

        if myThread.transaction.transaction == None:
            myThread.transaction.begin()
            return False

        return True

    def existingTransaction(self):
        """
        _existingTransaction_

        Return True if there is an open transaction, False otherwise.
        """
        myThread = threading.currentThread()

        if "transaction" not in dir(myThread):
            return False
        elif myThread.transaction.transaction != None:
            return True

        return False

    def commitTransaction(self, existingTransaction):
        """
        _commitTransaction_

        Commit a database transaction that was begun by self.beginTransaction().
        """
        if not existingTransaction:
            myThread = threading.currentThread()
            myThread.transaction.commit()

        return

    def __getstate__(self):
        """
        __getstate__

        The database connection information isn't pickleable, so we to kill that
        before we attempt to pickle.
        """
        self.dbi = None
        self.logger = None
        self.daofactory = None
        return self.__dict__


    def transactionContext(self):
        """
        Returns a transaction as a ContextManager

        Usage:
            with transactionContext():
                databaseCode1()
                databaseCode2()

        Equates to beginTransaction() followed by either
        commitTransaction or a rollback
        """
        existingTransaction = self.beginTransaction()
        try:
            yield existingTransaction
        except:
            # responsibility for rolling back is on the transaction starter
            if not existingTransaction:
                self.logger.error('Exception caught, rolling back transaction')
                threading.currentThread().transaction.rollback()
            raise
        else:
            # only commits if transaction started by this invocation
            self.commitTransaction(existingTransaction)
    try:
        transactionContext = contextmanager(transactionContext)
    except NameError:
        pass


    def rollbackTransaction(self, existingTransaction):
        """Rollback transaction if we started it"""
        if not existingTransaction:
            threading.currentThread().transaction.rollback()



#!/usr/bin/env python
"""
_WMConfigCache_

Being in itself a wrapped class around a config cache
"""

from future import standard_library
standard_library.install_aliases()
import urllib.request, urllib.parse, urllib.error
from builtins import str
from future.utils import with_metaclass
from future.utils import viewkeys, viewvalues

import hashlib
import logging
import traceback
from contextlib import closing

import WMCore.GroupUser.Decorators as Decorators
from Utils.Patterns import Singleton
from Utils.Utilities import decodeBytesToUnicode
from Utils.PythonVersion import PY3
from WMCore.DataStructs.WMObject import WMObject
from WMCore.Database.CMSCouch import CouchNotFoundError
from WMCore.Database.CMSCouch import CouchServer, Document
from WMCore.GroupUser.Group import Group
from WMCore.GroupUser.User import makeUser
from WMCore.WMException import WMException


class ConfigCacheException(WMException):
    """
    Placeholder for a smarter exception class


    """


class DocumentCache(with_metaclass(Singleton, object)):
    """DocumentCache holds config ids. Use this class as singleton"""

    def __init__(self, database, detail=True):
        super(DocumentCache, self).__init__()
        self.cache = {}
        self.database = database
        # flag to decide whether update detail or not when it is prefetched
        self.detail = detail

    def __getitem__(self, configID):
        """
        Internal get method to fetch document based on provided ID
        """
        if configID not in self.cache:
            self.prefetch([configID])
        return self.cache[configID]

    def cleanup(self, ids):
        """
        _cleanup_

        Clean-up given ids from local cache
        """
        for rid in ids:
            if rid in self.cache:
                del self.cache[rid]

    def prefetch(self, keys):
        """
        _fetch_

        Pre-fetch given list of documents from CouchDB
        """
        if self.detail:
            options = {'include_docs': True}
        else:
            options = {}
        result = self.database.allDocs(options=options, keys=keys)
        for row in result['rows']:
            if self.detail:
                self.cache[row['id']] = row['doc']
            else:
                self.cache[row['id']] = True


class ConfigCache(WMObject):
    """
    _ConfigCache_

    The class that handles the upload and download of configCache
    artifacts from Couch
    """

    def __init__(self, dbURL, couchDBName=None, id=None, rev=None, usePYCurl=True,
                 ckey=None, cert=None, capath=None, detail=True):
        super(ConfigCache, self).__init__()
        self.dbname = couchDBName
        self.dburl = dbURL
        self.detail = detail
        try:
            self.couchdb = CouchServer(self.dburl, usePYCurl=usePYCurl, ckey=ckey, cert=cert, capath=capath)
            if self.dbname not in self.couchdb.listDatabases():
                self.createDatabase()

            self.database = self.couchdb.connectDatabase(self.dbname)
        except Exception as ex:
            msg = "Error connecting to couch: %s\n" % str(ex)
            msg += str(traceback.format_exc())
            logging.error(msg)
            raise ConfigCacheException(message=msg)

        # local cache
        self.docs_cache = DocumentCache(self.database, self.detail)

        # UserGroup variables
        self.group = None
        self.owner = None

        # Internal data structure
        self.document = Document()
        self.attachments = {}
        self.document['type'] = "config"
        self.document['description'] = {}
        self.document['description']['config_label'] = None
        self.document['description']['config_desc'] = None

        if id != None:
            self.document['_id'] = id
        self.document['pset_tweak_details'] = None
        self.document['info'] = None
        self.document['config'] = None
        return

    def createDatabase(self):
        """
        _createDatabase_

        """
        database = self.couchdb.createDatabase(self.dbname)
        database.commit()
        return database

    def connectUserGroup(self, groupname, username):
        """
        _connectUserGroup_

        """
        self.group = Group(name=groupname)
        self.group.setCouch(self.dburl, self.dbname)
        self.group.connect()
        self.owner = makeUser(groupname, username,
                              couchUrl=self.dburl,
                              couchDatabase=self.dbname)
        return

    def createUserGroup(self, groupname, username):
        """
        _createUserGroup_

        Create all the userGroup information
        """
        self.createGroup(name=groupname)
        self.createUser(username=username)
        return

    def createGroup(self, name):
        """
        _createGroup_

        Create Group for GroupUser
        """
        self.group = Group(name=name)
        self.group.setCouch(self.dburl, self.dbname)
        self.group.connect()
        self.group.create()
        return

    def setLabel(self, label):
        """
        _setLabel_

        Util to add a descriptive label to the configuration doc
        """
        self.document['description']['config_label'] = label

    def setDescription(self, desc):
        """
        _setDescription_

        Util to add a verbose description string to a configuration doc
        """
        self.document['description']['config_desc'] = desc

    @Decorators.requireGroup
    def createUser(self, username):
        self.owner = makeUser(self.group['name'], username,
                              couchUrl=self.dburl,
                              couchDatabase=self.dbname)
        self.owner.create()
        self.owner.ownThis(self.document)
        return

    @Decorators.requireGroup
    @Decorators.requireUser
    def save(self):
        """
        _save_

        Save yourself!  Save your internal document.
        """
        rawResults = self.database.commit(doc=self.document)

        # We should only be committing one document at a time
        # if not, get the last one.

        try:
            commitResults = rawResults[-1]
            self.document["_rev"] = commitResults.get('rev')
            self.document["_id"] = commitResults.get('id')
        except KeyError as ex:
            msg = "Document returned from couch without ID or Revision\n"
            msg += "Document probably bad\n"
            msg += str(ex)
            logging.error(msg)
            raise ConfigCacheException(message=msg)

        # Now do the attachments
        for attachName in self.attachments:
            self.saveAttachment(name=attachName,
                                attachment=self.attachments[attachName])

        return

    def saveAttachment(self, name, attachment):
        """
        _saveAttachment_

        Save an attachment to the document
        """

        retval = self.database.addAttachment(self.document["_id"],
                                             self.document["_rev"],
                                             attachment,
                                             name)

        if retval.get('ok', False) != True:
            # Then we have a problem
            msg = "Adding an attachment to document failed\n"
            msg += str(retval)
            msg += "ID: %s, Rev: %s" % (self.document["_id"], self.document["_rev"])
            logging.error(msg)
            raise ConfigCacheException(msg)

        self.document["_rev"] = retval['rev']
        self.document["_id"] = retval['id']
        self.attachments[name] = attachment

        return

    def loadDocument(self, configID):
        """
        _loadDocument_

        Load a document from the document cache given its couchID
        """
        self.document = self.docs_cache[configID]

    def loadByID(self, configID):
        """
        _loadByID_

        Load a document from the server given its couchID
        """
        try:
            self.document = self.database.document(id=configID)
            if 'owner' in self.document:
                self.connectUserGroup(groupname=self.document['owner'].get('group', None),
                                      username=self.document['owner'].get('user', None))
            if '_attachments' in self.document:
                # Then we need to load the attachments
                for key in viewkeys(self.document['_attachments']):
                    self.loadAttachment(name=key)
        except CouchNotFoundError as ex:
            msg = "Document with id %s not found in couch\n" % (configID)
            msg += str(ex)
            msg += str(traceback.format_exc())
            logging.error(msg)
            raise ConfigCacheException(message=msg)
        except Exception as ex:
            msg = "Error loading document from couch\n"
            msg += str(ex)
            msg += str(traceback.format_exc())
            logging.error(msg)
            raise ConfigCacheException(message=msg)

        return

    def loadAttachment(self, name, overwrite=True):
        """
        _loadAttachment_

        Load an attachment from the database and put it somewhere useful
        """

        attach = self.database.getAttachment(self.document["_id"], name)

        if not overwrite:
            if name in self.attachments:
                logging.info("Attachment already exists, so we're skipping")
                return

        self.attachments[name] = attach

        return

    def loadByView(self, view, value):
        """
        _loadByView_

        Underlying code to load views
        """

        viewRes = self.database.loadView('ConfigCache', view, {}, [value])

        if not viewRes['rows']:
            # Then we have a problem
            msg = "Unable to load using view %s and value %s" % (view, str(value))
            logging.error(msg)

        self.unwrapView(viewRes)
        self.loadByID(self.document["_id"])
        return

    def saveConfigToDisk(self, targetFile):
        """
        _saveConfigToDisk_

        Make sure we can save our config file to disk
        """
        config = self.getConfig()
        if not config:
            return

        # Write to a file
        with open(targetFile, 'w') as f:
            f.write(config)
        return

    def load(self):
        """
        _load_

        Figure out how to load
        """

        if self.document.get("_id", None) != None:
            # Then we should load by ID
            self.loadByID(self.document["_id"])
            return

        # Otherwise we have to load by view

        if not self.document.get('md5_hash', None) is None:
            # Then we have an md5_hash
            self.loadByView(view='config_by_md5hash', value=self.document['md5_hash'])
            # TODO: Add more views as they become available.


            # elif not self.owner == None:
            # Then we have an owner
            # self.loadByView(view = 'config_by_owner', value = self.owner['name'])

    def unwrapView(self, view):
        """
        _unwrapView_

        Move view information into the main document
        """

        self.document["_id"] = view['rows'][0].get('id')
        self.document["_rev"] = view['rows'][0].get('value').get('_rev')

    def setPSetTweaks(self, PSetTweak):
        """
        _setPSetTweaks_

        Set the PSet tweak details for the config.
        """
        self.document['pset_tweak_details'] = PSetTweak
        return

    def getPSetTweaks(self):
        """
        _getPSetTweaks_

        Retrieve the PSet tweak details.
        """
        return self.document['pset_tweak_details']

    def getOutputModuleInfo(self):
        """
        _getOutputModuleInfo_

        Retrieve the dataset information for the config in the ConfigCache.
        """
        psetTweaks = self.getPSetTweaks()
        if 'process' not in psetTweaks:
            raise ConfigCacheException("Could not find process field in PSet while getting output modules!")
        try:
            outputModuleNames = psetTweaks["process"]["outputModules_"]
        except KeyError as ex:
            msg = "Could not find outputModules_ in psetTweaks['process'] while getting output modules.\n"
            msg += str(ex)
            logging.error(msg)
            raise ConfigCacheException(msg)

        results = {}
        for outputModuleName in outputModuleNames:
            try:
                outModule = psetTweaks["process"][outputModuleName]
            except KeyError:
                msg = "Could not find outputModule %s in psetTweaks['process']" % outputModuleName
                logging.error(msg)
                raise ConfigCacheException(msg)
            dataset = outModule.get("dataset", None)
            if dataset:
                results[outputModuleName] = {"dataTier": outModule["dataset"]["dataTier"],
                                             "filterName": outModule["dataset"]["filterName"]}
            else:
                results[outputModuleName] = {"dataTier": None, "filterName": None}

        return results

    def addConfig(self, newConfig, psetHash=None):
        """
        _addConfig_


        """
        # The newConfig parameter is a URL suitable for passing to urlopen.
        with closing(urllib.request.urlopen(newConfig)) as f:
            configString = f.read(-1)
        configMD5 = hashlib.md5(configString).hexdigest()
        if PY3:
            configString = decodeBytesToUnicode(configString)

        self.document['md5_hash'] = configMD5
        self.document['pset_hash'] = psetHash
        self.attachments['configFile'] = configString
        return

    def getConfig(self):
        """
        _getConfig_

        Get the currently active config
        """
        return self.attachments.get('configFile', None)

    def getCouchID(self):
        """
        _getCouchID_

        Return the document's couchID
        """

        return self.document["_id"]

    def getCouchRev(self):
        """
        _getCouchRev_

        Return the document's couchRevision
        """

        return self.document["_rev"]

    @Decorators.requireGroup
    @Decorators.requireUser
    def delete(self):
        """
        _delete_

        Deletes the document with the current docid
        """
        if not self.document["_id"]:
            logging.error("Attempted to delete with no couch ID")

        # TODO: Delete without loading first
        try:
            self.database.queueDelete(self.document)
            self.database.commit()
        except Exception as ex:
            msg = "Error in deleting document from couch"
            msg += str(ex)
            msg += str(traceback.format_exc())
            logging.error(msg)
            raise ConfigCacheException(message=msg)

        return

    def getIDFromLabel(self, label):
        """
        _getIDFromLabel_

        Retrieve the ID of a config given it's label.
        """
        results = self.database.loadView("ConfigCache", "config_by_label",
                                         {"startkey": label,
                                          "limit": 1})

        if results["rows"][0]["key"] == label:
            return results["rows"][0]["value"]

        return None

    def listAllConfigsByLabel(self):
        """
        _listAllConfigsByLabel_

        Retrieve a list of all the configs in the config cache.  This is
        returned in the form of a dictionary that is keyed by label.
        """
        configs = {}
        results = self.database.loadView("ConfigCache", "config_by_label")

        for result in results["rows"]:
            configs[result["key"]] = result["value"]

        return configs

    def __str__(self):
        """
        Make something printable

        """

        return self.document.__str__()

    def validate(self, configID):

        try:
            # TODO: need to change to DataCache
            # self.loadDocument(configID = configID)
            self.loadByID(configID=configID)
        except Exception as ex:
            raise ConfigCacheException("Failure to load ConfigCache while validating workload: %s" % str(ex))

        if self.detail:
            duplicateCheck = {}
            try:
                outputModuleInfo = self.getOutputModuleInfo()
            except Exception as ex:
                # Something's gone wrong with trying to open the configCache
                msg = "Error in getting output modules from ConfigCache during workload validation.  Check ConfigCache formatting!"
                raise ConfigCacheException("%s: %s" % (msg, str(ex)))
            for outputModule in viewvalues(outputModuleInfo):
                dataTier = outputModule.get('dataTier', None)
                filterName = outputModule.get('filterName', None)
                if not dataTier:
                    raise ConfigCacheException("No DataTier in output module.")

                # Add dataTier to duplicate dictionary
                if dataTier not in duplicateCheck:
                    duplicateCheck[dataTier] = []
                if filterName in duplicateCheck[dataTier]:
                    # Then we've seen this combination before
                    raise ConfigCacheException("Duplicate dataTier/filterName combination.")
                else:
                    duplicateCheck[dataTier].append(filterName)
            return outputModuleInfo
        return True



"""
File       : RestApiHub.py
Author     : Valentin Kuznetsov <vkuznet AT gmail dot com>
Description: REST API module for MicroService class.
"""

# futures
from __future__ import print_function, division

# cherrypy modules
import cherrypy

# WMCore modules
from WMCore.Configuration import Configuration
from WMCore.REST.Server import RESTApi
from WMCore.MicroService.Service.Data import Data


class RestApiHub(RESTApi):
    """
    RestInterface defines REST APIs for MicroService.
    They are mounted to entry point defined in _add method.
    """
    def __init__(self, app, config, mount):
        """
        :arg app: reference to application object; passed to all entities.
        :arg config: reference to configuration; passed to all entities.
        :arg str mount: is definition coming from configuration file to mount the app in REST server
        """

        RESTApi.__init__(self, app, config, mount)

        cherrypy.log("MicroService entire configuration:\n%s" % Configuration.getInstance())
        cherrypy.log("MicroService REST configuration subset:\n%s" % config)

        self._add({"status": Data(app, self, config, mount),
                   "info": Data(app, self, config, mount)})

        # the mount point should match service configuration, e.g. in MSPileup we define
        # mount point as /ms-pileup/data, e.g. http://.../ms-pileup/data
        if 'ms-pileup' in mount:
            self._add({"pileup": Data(app, self, config, mount)})



#!/usr/bin/env python
# encoding: utf-8
"""
WMBase.py

Created by Dave Evans on 2011-05-20.
Copyright (c) 2011 Fermilab. All rights reserved.
"""

import sys
import os
import os.path
import inspect

def getWMBASE():
    """ returns the root of WMCore install """
    if __file__.find("src/python") != -1:
        return os.path.normpath(os.path.join(os.path.dirname(__file__), '..', '..', '..'))
    else:
        return os.path.normpath(os.path.join(os.path.dirname(__file__), '..'))

def getTestBase(importFlag = True):
    """
    _getTestBase_

    Returns a base that can be used for testing.  Defaults to
    getWMBase if no environment variables WMCORE_TEST_ROOT is defined
    """
    basePath = os.path.normpath(os.path.join(getWMBASE(), 'test/python'))
    envPath  = os.environ.get("WMCORE_TEST_ROOT", None)

    # First, try getting things from the environment
    if envPath != None:
        try:
            if os.path.isdir(envPath):
                return envPath
        except:
            pass

    if importFlag:
        # Then try importing things from WMCore_t and see if we can
        # find the directory
        try:
            import WMCore_t.__init__ as testImport
            testPath = os.path.dirname(inspect.getsourcefile(testImport))
            return os.path.normpath(os.path.join(testPath, '../'))
        except ImportError:
            pass

    return basePath
